#!/usr/bin/env bash
# ------------------------------------------------------------------------------
# This file contains helper functions usefull in other scripts.
# Note that this file needs to be sourced to have effect!
# ------------------------------------------------------------------------------

# Global color definitions.
NC='\033[0m'
RED='\033[1;31m'
GREEN='\033[1;32m'
YELLOW='\033[1;33m'

# Display fatal styled message with given argument.
function fatal {
    echo -e "$RED \u274c FATAL: $1$NC" 1>&2
    return 1
}

# Display success styled message with given argument.
function success {
    echo -e "$GREEN \u2714 ~ $1$NC" 1>&2
}

# Display info styled message with given argument.
function info {
    echo -e "$YELLOW \u003F ~ $1$NC" 1>&2
}

# Check if PLATFORM is valid.
function platform_check {
    IFS="|"
    options=("lumi${IFS}mn5")

    if [[ -z "${PLATFORM}" ]]; then
        fatal "No platform selected. Options: [ lumi | mn5 ]"
        exit $?
    elif [[ ! "${IFS}${options[*]}${IFS}" =~ "${IFS}${PLATFORM}${IFS}" ]]; then
        fatal "Platform '${PLATFORM}' unknown. Options: [ lumi | mn5 ]"
        exit $?
    fi
}
