#!/usr/bin/env bash
# ------------------------------------------------------------------------------
# This file sets up the environment correctly for building on the provided 
# platform.
# Note that this file needs to be sourced to have effect!
# ------------------------------------------------------------------------------

# The BASEMOD is the PWD of profile.sh, regardless of the callers location.
export BASEMOD=$(cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd)

# Source helpers for colored prints.
source $BASEMOD/helpers.sh


# Load modules based on platform.
echo ""
if [[ -z "${PLATFORM}" ]]; then
    fatal "No platform selected. Options: [ lumi | mn5 ]"
elif [ "$PLATFORM" == "lumi" ]; then
    info "Loading modules for Lumi.."
    module purge
    module load LUMI/23.03 partition/G PrgEnv-cray cpe/23.03 craype-x86-trento \
        craype-accel-amd-gfx90a rocm
elif [ "$PLATFORM" == "mn5" ]; then
    info "Loading modules for MareNostrum 5.."
    # TODO: Load MN5 modules.
else
    fatal "Platform '${PLATFORM}' unknown. Options: [ lumi | mn5 ]"
fi
