! Main solver routines for an inefficient heat equation solver.
module core_gpu_good
    use heat

    contains

    ! Compute one time step of temperature evolution.
    ! Arguments:
    !   curr    (type(field)):  Current temperature values.
    !   prev    (type(field)):  Values from previous time step.
    !   a       (real(dp)):     Update equation constant.
    !   dt      (real(dp)):     Time step value.
    subroutine evolve(curr, prev, a, dt)
        implicit none
    
        type(field),target, intent(inout) :: curr, prev
        real(dp) :: a, dt
        integer :: i, j, nx, ny
        real(dp) :: dx, dy
        real(dp), pointer, contiguous, dimension(:,:) :: currdata, prevdata
    
        ! Help the compiler avoid being confused.
        nx = curr%nx
        ny = curr%ny
        dx = curr%dx
        dy = curr%dy
        currdata => curr%data
        prevdata => prev%data

        !$omp target teams distribute parallel do
        do j = 1, ny
            do i = 1, nx
                currdata(i, j) = prevdata(i, j) + a * dt * &
                    & ((prevdata(i-1, j) - 2.0 * prevdata(i, j) + &
                    &   prevdata(i+1, j)) / dx**2 + &
                    &  (prevdata(i, j-1) - 2.0 * prevdata(i, j) + &
                    &   prevdata(i, j+1)) / dy**2)
            end do
        end do
        !$omp end target teams distribute parallel do
    end subroutine evolve

end module core_gpu_good
