! Main solver routines for an inefficient heat equation solver.
module core_gpu_sloppy
    use heat

    contains

    ! Compute one time step of temperature evolution.
    ! Arguments:
    !   curr    (type(field)):  Current temperature values.
    !   prev    (type(field)):  Values from previous time step.
    !   a       (real(dp)):     Update equation constant.
    !   dt      (real(dp)):     Time step value.
    subroutine evolve(curr, prev, a, dt)
        implicit none

        type(field), intent(inout) :: curr, prev
        real(dp) :: a, dt
        integer :: i, j, nx, ny

        nx = curr%nx
        ny = curr%ny

        !$omp target teams distribute parallel do collapse(2)
        do j = 1, ny
            do i = 1, nx
                curr%data(i, j) = prev%data(i, j) + a * dt * &
                    & ((prev%data(i-1, j) - 2.0 * prev%data(i, j) + &
                    &   prev%data(i+1, j)) / curr%dx**2 + &
                    &  (prev%data(i, j-1) - 2.0 * prev%data(i, j) + &
                    &   prev%data(i, j+1)) / curr%dy**2)
            end do
        end do
        !$omp end target teams distribute parallel do
    end subroutine evolve

end module core_gpu_sloppy
