! Sloppy GPU heat equation solver in 2D.

program heat_solve_gpu_sloppy
    use heat
    use core_gpu_sloppy
    use setup
    use utilities
    use omp_lib

    implicit none

    real(dp), parameter :: a = 0.5      ! Diffusion constant.
    type(field) :: current, previous    ! Current and previus temperature fields.
    real(dp) :: dt                      ! Time step.
    integer :: nsteps                   ! Number of time steps.

    integer :: iter                     ! Counter of iterations for loop.
    real(dp) :: average_temp            ! Average temperature
    real(kind=dp) :: start, stop        ! Timers

    ! Initialize the field.
    call initialize(current, previous, nsteps)
    average_temp = average(current)
    write(*, '(A,F9.6)') 'Average temperature at start: ', average_temp

    ! Largest stable time step
    dt = current%dx**2 * current%dy**2 / &
         & (2.0 * a * (current%dx**2 + current%dy**2))

    ! Main iteration loop, save a picture every
    ! image_interval steps
    start =  omp_get_wtime()

    do iter = 1, nsteps
       call evolve(current, previous, a, dt)
       call swap_fields(current, previous)
    end do

    stop = omp_get_wtime()

    ! Average temperature for reference
    average_temp = average(previous)

    write(*, '(A30,F6.3,A)') 'Iterations took: ', stop - start, '    seconds.'
    write(*, '(A30,F9.6)') 'Average temperature: ',  average_temp
    if (command_argument_count() == 0) then
        write(*, '(A30,F9.6)') 'Reference value for default: ', 59.281239
    end if

    call finalize(current, previous)

end program heat_solve_gpu_sloppy
